#include <stdlib.h>
#include <string.h>

#include "buffer.h"

int8_t
buffer_add (Buffer * b, void * data, size_t n)
{
  while (b->len + n > b->size)
    {
      if (buffer_expand (b) == -1)
        return -1;
    }

  memmove ((void *) (b->buf + b->len), data, n);
  b->len += n;
  return 0;
}

void
buffer_clear (Buffer * b)
{
  b->len = 0;
  b->offset = 0;
}

int8_t
buffer_expand (Buffer * b)
{
  char * buf;

  buf = (char *) realloc (b->buf, (b->size + 512) * sizeof (char));
  if (buf == NULL)
    return -1;

  b->buf = buf;
  b->size += 512;

  return 0;
}

void
buffer_free (Buffer * b)
{
  free (b->buf);
  b->len = 0;
  b->offset = 0;
  b->size = 0;
}

void
buffer_get (Buffer * b, void * data, size_t n)
{
  char * buf = (char *) data;
  size_t i;

  for (i = 0; i < n; i++)
      buf[i] = b->buf[b->offset+i];

  b->offset += n;
}

int8_t
buffer_init (Buffer * b)
{
  b->buf = (char *) malloc (512 * sizeof (char));
  if (b->buf == NULL)
    return -1;

  b->len = 0;
  b->offset = 0;
  b->size = 512 * sizeof (char);
  return 0;
}

ssize_t
buffer_read (Buffer * b, int fd)
{
  char buf[512];
  size_t n;

  do
    {
      n = read (fd, (void *) buf, 512);
      buffer_add (b, (void *) buf, n);
    } while (n == 512);

  return n;
}

ssize_t
buffer_write (Buffer * b, int fd)
{
  return write (fd, (void *) b->buf, b->len);
}
